from . import profiler_output_pb2 as profiler_output
import inspect
import os
import psutil
import hashlib

def get_function_name(func):
    return func.__name__

def get_func_params(args, func):
    result = []
    sig = inspect.signature(func)
    params = sig.parameters
    for i, arg in enumerate(args):
        arg_name = list(params.keys())[i] if i < len(params) else 'N/A'
        result.append(profiler_output.FuncParams(
            arg=str(arg),
            arg_name=arg_name,
            type=type(arg).__name__
        ))
    return result

def get_memory_usage():
    process = psutil.Process(os.getpid())
    return process.memory_info().rss

def get_cpu_usage():
    process = psutil.Process(os.getpid())
    return process.cpu_percent()

def hash_function(func):
    try:
        source_code = inspect.getsource(func)
    except TypeError:
        raise ValueError("The provided argument is not a function.")

    source_hash = hashlib.sha256(source_code.encode('utf-8')).hexdigest()
    return source_hash

def get_caller():
    if len(inspect.stack()) < 3:
        return "N/A"
    caller_frame = inspect.stack()[2]
    caller_function = caller_frame.function
    return caller_function
